@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2025 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -23,21 +23,41 @@ import static java.util.ServiceLoader.load;
  */
 public class SessionManager
 {
-	private static Session s_session;
-
 	/**
 	 * Returns the current session.
 	 */
 	public static Session current()
 	throws SQLException
 	{
-		if(s_session == null)
+		try
 		{
-			s_session = load(
-				Session.class.getModule().getLayer(), Session.class)
-				.findFirst().orElseThrow(() -> new SQLException(
-					"could not obtain PL/Java Session object"));
+			return Holder.s_session;
+		}
+		catch ( ExceptionInInitializerError e )
+		{
+			Throwable c = e.getCause();
+			if ( c instanceof SQLException )
+				throw (SQLException)c;
+			throw e;
+		}
+	}
+
+	private static class Holder
+	{
+		private static final Session s_session;
+
+		static {
+			try
+			{
+				s_session = load(
+					Session.class.getModule().getLayer(), Session.class)
+					.findFirst().orElseThrow(() -> new SQLException(
+						"could not obtain PL/Java Session object"));
+			}
+			catch ( SQLException e )
+			{
+				throw new ExceptionInInitializerError(e);
+			}
 		}
-		return s_session;
 	}
 }
