@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2025 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -127,20 +127,26 @@ public class SPIActions {
 		}
 	}
 
-	static void log(String msg) {
+	static void log(String msg) throws SQLException {
 		// GCJ has a somewhat serious bug (reported)
 		//
-		if ("GNU libgcj".equals(System.getProperty("java.vm.name"))) {
+		if ("GNU libgcj"
+				.equals(
+					SessionManager.current().frozenSystemProperties()
+						.getProperty("java.vm.name"))) {
 			System.out.print("INFO: ");
 			System.out.println(msg);
 		} else
 			Logger.getAnonymousLogger().info(msg);
 	}
 
-	static void warn(String msg) {
+	static void warn(String msg) throws SQLException {
 		// GCJ has a somewhat serious bug (reported)
 		//
-		if ("GNU libgcj".equals(System.getProperty("java.vm.name"))) {
+		if ("GNU libgcj"
+				.equals(
+					SessionManager.current().frozenSystemProperties()
+						.getProperty("java.vm.name"))) {
 			System.out.print("WARNING: ");
 			System.out.println(msg);
 		} else
