@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2022 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2023 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -34,6 +34,10 @@
 #include <funcapi.h>
 #include <utils/typcache.h>
 
+#if PG_VERSION_NUM >= 160000
+#define PG_FUNCNAME_MACRO __func__
+#endif
+
 #ifdef _MSC_VER
 #	define strcasecmp _stricmp
 #	define strncasecmp _strnicmp
@@ -41,15 +45,6 @@
 
 #define PARAM_OIDS(procStruct) (procStruct)->proargtypes.values
 
-#if 90305<=PG_VERSION_NUM || \
-	90209<=PG_VERSION_NUM && PG_VERSION_NUM<90300 || \
-	90114<=PG_VERSION_NUM && PG_VERSION_NUM<90200 || \
-	90018<=PG_VERSION_NUM && PG_VERSION_NUM<90100 || \
-	80422<=PG_VERSION_NUM && PG_VERSION_NUM<90000
-#else
-#error "Need fallback for heap_copy_tuple_as_datum"
-#endif
-
 #define COUNTCHECK(refs, prims) ((jshort)(((refs) << 8) | ((prims) & 0xff)))
 
 jobject pljava_Function_NO_LOADER;
