@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -76,19 +76,21 @@ static HeapTuple _getTupleAndClear(jobject jrps)
  * true. If so, the values are obtained in the form of a HeapTuple which in
  * turn is returned (as a Datum) from this method.
  */
-static Datum _Composite_invoke(Type self, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
+static Datum _Composite_invoke(Type self, Function fn, PG_FUNCTION_ARGS)
 {
 	bool hasRow;
 	Datum result = 0;
 	TupleDesc tupleDesc = Type_getTupleDesc(self, fcinfo);
 	jobject jtd = pljava_TupleDesc_create(tupleDesc);
-	jobject singleRowWriter = _createWriter(jtd);
-	int numArgs = fcinfo->nargs;
+	jvalue singleRowWriter;
+	singleRowWriter.l = _createWriter(jtd);
+	/*
+	 * Caller guarantees room for one extra reference parameter, so it will go
+	 * at index (length - 1).
+	 */
+	pljava_Function_setParameter(fn, -1, singleRowWriter);
 	
-	// Caller guarantees room for one extra slot
-	//
-	args[numArgs].l = singleRowWriter;
-	hasRow = (JNI_callStaticBooleanMethodA(cls, method, args) == JNI_TRUE);
+	hasRow = (pljava_Function_booleanInvoke(fn) == JNI_TRUE);
 
 	if(hasRow)
 	{
@@ -96,7 +98,7 @@ static Datum _Composite_invoke(Type self, jclass cls, jmethodID method, jvalue*
 		 * durable context.
 		 */
 		MemoryContext currCtx = Invocation_switchToUpperContext();
-		HeapTuple tuple = _getTupleAndClear(singleRowWriter);
+		HeapTuple tuple = _getTupleAndClear(singleRowWriter.l);
 	    result = HeapTupleGetDatum(tuple);
 		MemoryContextSwitchTo(currCtx);
 	}
@@ -104,22 +106,10 @@ static Datum _Composite_invoke(Type self, jclass cls, jmethodID method, jvalue*
 		fcinfo->isnull = true;
 
 	JNI_deleteLocalRef(jtd);
-	JNI_deleteLocalRef(singleRowWriter);
+	JNI_deleteLocalRef(singleRowWriter.l);
 	return result;
 }
 
-static jobject _Composite_getSRFProducer(Type self, jclass cls, jmethodID method, jvalue* args)
-{
-	jobject tmp = JNI_callStaticObjectMethodA(cls, method, args);
-	if(tmp != 0 && JNI_isInstanceOf(tmp, s_ResultSetHandle_class))
-	{
-		jobject wrapper = JNI_newObject(s_ResultSetPicker_class, s_ResultSetPicker_init, tmp);
-		JNI_deleteLocalRef(tmp);
-		tmp = wrapper;
-	}
-	return tmp;
-}
-
 static jobject _Composite_getSRFCollector(Type self, PG_FUNCTION_ARGS)
 {
 	jobject tmp1;
@@ -134,23 +124,8 @@ static jobject _Composite_getSRFCollector(Type self, PG_FUNCTION_ARGS)
 	return tmp2;
 }
 
-static bool _Composite_hasNextSRF(Type self, jobject rowProducer, jobject rowCollector, jlong callCounter)
-{
-	/* Obtain next row using the RowCollector as a parameter to the
-	 * ResultSetProvider.assignRowValues method.
-	 */
-	if ( callCounter > PG_INT32_MAX )
-		ereport(ERROR,
-				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("the ResultSetProvider cannot return more than "
-					"INT32_MAX rows")));
-	return (JNI_callBooleanMethod(rowProducer,
-			s_ResultSetProvider_assignRowValues,
-			rowCollector,
-			(jint)callCounter) == JNI_TRUE);
-}
-
-static Datum _Composite_nextSRF(Type self, jobject rowProducer, jobject rowCollector)
+static Datum _Composite_datumFromSRF(
+	Type self, jobject row, jobject rowCollector)
 {
 	Datum result = 0;
 	HeapTuple tuple = _getTupleAndClear(rowCollector);
@@ -159,11 +134,6 @@ static Datum _Composite_nextSRF(Type self, jobject rowProducer, jobject rowColle
 	return result;
 }
 
-static void _Composite_closeSRF(Type self, jobject rowProducer)
-{
-	JNI_callVoidMethod(rowProducer, s_ResultSetProvider_close);
-}
-
 /* Assume that the Datum is a HeapTupleHeader and convert it into
  * a SingleRowReader instance.
  */
@@ -227,15 +197,6 @@ static TupleDesc _Composite_getTupleDesc(Type self, PG_FUNCTION_ARGS)
 	return td;
 }
 
-static const char* _Composite_getJNIReturnSignature(Type self, bool forMultiCall, bool useAltRepr)
-{
-	return forMultiCall
-		? (useAltRepr
-			? "Lorg/postgresql/pljava/ResultSetHandle;"
-			: "Lorg/postgresql/pljava/ResultSetProvider;")
-		: "Z";
-}
-
 Type Composite_obtain(Oid typeId)
 {
 	Composite infant = (Composite)TypeClass_allocInstance(s_CompositeClass, typeId);
@@ -273,12 +234,8 @@ void Composite_initialize(void)
 	s_CompositeClass->getTupleDesc    = _Composite_getTupleDesc;
 	s_CompositeClass->coerceDatum     = _Composite_coerceDatum;
 	s_CompositeClass->invoke          = _Composite_invoke;
-	s_CompositeClass->getSRFProducer  = _Composite_getSRFProducer;
 	s_CompositeClass->getSRFCollector = _Composite_getSRFCollector;
-	s_CompositeClass->hasNextSRF      = _Composite_hasNextSRF;
-	s_CompositeClass->nextSRF         = _Composite_nextSRF;
-	s_CompositeClass->closeSRF        = _Composite_closeSRF;
-	s_CompositeClass->getJNIReturnSignature = _Composite_getJNIReturnSignature;
+	s_CompositeClass->datumFromSRF    = _Composite_datumFromSRF;
 	s_CompositeClass->outParameter    = true;
 
 	Type_registerType2(InvalidOid, "java.sql.ResultSet", Composite_obtain);
