@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2021 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2023 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -9,6 +9,7 @@
  * Contributors:
  *   Tada AB - Thomas Hallgren
  *   Chapman Flack
+ *   Francisco Miguel Biete Banon
  */
 #include "pljava/type/String_priv.h"
 #include "pljava/HashMap.h"
@@ -59,9 +60,9 @@ jvalue _String_coerceDatum(Type self, Datum arg)
 {
 	jvalue result;
 	char* tmp = DatumGetCString(FunctionCall3(
-					&((String)self)->textOutput,
+					&((PLJString)self)->textOutput,
 					arg,
-					ObjectIdGetDatum(((String)self)->elementType),
+					ObjectIdGetDatum(((PLJString)self)->elementType),
 					Int32GetDatum(-1)));
 	result.l = String_createJavaStringFromNTS(tmp);
 	pfree(tmp);
@@ -83,19 +84,19 @@ Datum _String_coerceObject(Type self, jobject jstr)
 	JNI_deleteLocalRef(jstr);
 
 	ret = FunctionCall3(
-					&((String)self)->textInput,
+					&((PLJString)self)->textInput,
 					CStringGetDatum(tmp),
-					ObjectIdGetDatum(((String)self)->elementType),
+					ObjectIdGetDatum(((PLJString)self)->elementType),
 					Int32GetDatum(-1));
 	pfree(tmp);
 	return ret;
 }
 
-static String String_create(TypeClass cls, Oid typeId)
+static PLJString String_create(TypeClass cls, Oid typeId)
 {
 	HeapTuple    typeTup = PgObject_getValidTuple(TYPEOID, typeId, "type");
 	Form_pg_type pgType  = (Form_pg_type)GETSTRUCT(typeTup);
-	String self = (String)TypeClass_allocInstance(cls, typeId);
+	PLJString self = (PLJString)TypeClass_allocInstance(cls, typeId);
 	MemoryContext ctx = GetMemoryChunkContext(self);
 	fmgr_info_cxt(pgType->typoutput, &self->textOutput, ctx);
 	fmgr_info_cxt(pgType->typinput,  &self->textInput,  ctx);
@@ -109,7 +110,7 @@ Type String_obtain(Oid typeId)
 	return (Type)StringClass_obtain(s_StringClass, typeId);
 }
 
-String StringClass_obtain(TypeClass self, Oid typeId)
+PLJString StringClass_obtain(TypeClass self, Oid typeId)
 {
 	return String_create(self, typeId);
 }
@@ -126,7 +127,7 @@ jstring String_createJavaString(text* t)
 		Size srcLen = VARSIZE(t) - VARHDRSZ;
 		if(srcLen == 0)
 			return s_the_empty_string;
-	
+
 		if ( s_two_step_conversion )
 		{
 			utf8 = (char*)pg_do_encoding_conversion((unsigned char*)src,
@@ -225,11 +226,7 @@ text* String_createText(jstring javaString)
 		/* Allocate and initialize the text structure.
 		 */
 		result = (text*)palloc(varSize);
-#if PG_VERSION_NUM < 80300
-		VARATT_SIZEP(result) = varSize;	/* Total size of structure, not just data */
-#else
 		SET_VARSIZE(result, varSize);	/* Total size of structure, not just data */
-#endif
 		memcpy(VARDATA(result), denc, dencLen);
 
 		if(denc != sid.data)
@@ -422,6 +419,12 @@ static void String_initialize_codec()
 	jclass buffer_class = PgObject_getJavaClass("java/nio/Buffer");
 	jobject servercs;
 
+	/*
+	 * Records what the final state of s_two_step_conversion will be, but the
+	 * static is left at its initial value until all preparations are complete.
+	 */
+	bool two_step_when_ready = s_two_step_conversion;
+
 	s_server_encoding = GetDatabaseEncoding();
 
 	if ( PG_SQL_ASCII == s_server_encoding )
@@ -431,7 +434,7 @@ static void String_initialize_codec()
 				"forName", "(Ljava/lang/String;)Ljava/nio/charset/Charset;");
 		jstring sql_ascii = JNI_newStringUTF("X-PGSQL_ASCII");
 
-		s_two_step_conversion = false;
+		two_step_when_ready = false;
 
 		servercs = JNI_callStaticObjectMethodLocked(charset_class,
 			forname, sql_ascii);
@@ -443,7 +446,7 @@ static void String_initialize_codec()
 		jfieldID scharset_UTF_8 = PgObject_getStaticJavaField(scharset_class,
 			"UTF_8", "Ljava/nio/charset/Charset;");
 
-		s_two_step_conversion = PG_UTF8 != s_server_encoding;
+		two_step_when_ready = PG_UTF8 != s_server_encoding;
 
 		servercs = JNI_getStaticObjectField(scharset_class, scharset_UTF_8);
 	}
@@ -477,5 +480,6 @@ static void String_initialize_codec()
 	s_the_empty_string = JNI_newGlobalRef(
 		JNI_callObjectMethod(empty, string_intern));
 
+	s_two_step_conversion = two_step_when_ready;
 	uninitialized = false;
 }
