@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2021 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -52,9 +52,29 @@ struct Invocation_
 	bool          inExprContextCB;
 
 	/**
-	 * Set to true if the executing function is trusted
+	 * The saved limits reserved in Function.c's static parameter frame, as a
+	 * count of reference and primitive parameters combined in a short.
+	 * FRAME_LIMITS_PUSHED is an otherwise invalid value used to record that the
+	 * more heavyweight saving of the frame as a Java ParameterFrame instance
+	 * has occurred. Otherwise, this value (and the primitive slot 0 value
+	 * below) are simply restored when this Invocation is exited normally or
+	 * exceptionally.
 	 */
-	bool          trusted;
+	jshort frameLimits;
+#define FRAME_LIMITS_PUSHED ((jshort)-1)
+
+	/**
+	 * The saved value of the first primitive slot in Function's static
+	 * parameter frame. Unless frameLimits above is FRAME_LIMITS_PUSHED, this
+	 * value is simply restored when this Invocation is exited normally or
+	 * exceptionally.
+	 */
+	jvalue primSlot0;
+
+	/**
+	 * The saved thread context classloader from before this invocation
+	 */
+	jobject savedLoader;
 
 	/**
 	 * The currently executing Function.
@@ -96,10 +116,17 @@ extern void Invocation_pushBootContext(Invocation* ctx);
 
 extern void Invocation_popBootContext(void);
 
-extern void Invocation_pushInvocation(Invocation* ctx, bool trusted);
+extern void Invocation_pushInvocation(Invocation* ctx);
 
 extern void Invocation_popInvocation(bool wasException);
 
+/*
+ * Return the type map held by the innermost executing PL/Java function's
+ * schema loader (the initiating loader that was used to resolve the function).
+ * The type map is a map from Java Oid objects to Class<SQLData> class objects,
+ * as resolved by that loader. This is effectively Function_currentLoader()
+ * followed by JNI-invoking getTypeMap on the loader, but cached to avoid JNI).
+ */
 extern jobject Invocation_getTypeMap(void);
 
 /*
@@ -112,6 +139,13 @@ extern jobject Invocation_getTypeMap(void);
  */
 extern MemoryContext Invocation_switchToUpperContext(void);
 
+/*
+ * Called only during Function's initialization to supply these values, making
+ * them cheap to access during pushInvocation/popInvocation, while still a bit
+ * more encapsulated than if they were made global.
+ */
+extern void pljava_Invocation_shareFrame(jvalue *slot0, jshort *limits);
+
 #ifdef __cplusplus
 }
 #endif
