@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2023 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2025 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -146,7 +146,11 @@ extern MemoryContext JavaMemoryContext;
  * 
  * Class 07 - Dynamic SQL Exception
  */
-#define ERRCODE_INVALID_DESCRIPTOR_INDEX		MAKE_SQLSTATE('0','7', '0','0','9')
+#define ERRCODE_INVALID_DESCRIPTOR_INDEX MAKE_SQLSTATE('0','7', '0','0','9')
+/*
+ * Class 46 - SQL/JRT
+ */
+#define ERRCODE_CLASS_SQLJRT MAKE_SQLSTATE('4','6','0','0','0')
 
 /*
  * Union used when coercing void* to jlong and vice versa
