@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2018 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -15,7 +15,6 @@
 #ifndef __pljava_Tuple_h
 #define __pljava_Tuple_h
 
-#include "pljava/type/JavaWrapper.h"
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -32,16 +31,18 @@ extern "C" {
 /*
  * Create the org.postgresql.pljava.Tuple instance
  */
-extern jobject Tuple_create(HeapTuple tuple);
-extern jobject Tuple_internalCreate(HeapTuple tuple, bool mustCopy);
-extern jobjectArray Tuple_createArray(HeapTuple* tuples, jint size, bool mustCopy);
+extern jobject pljava_Tuple_create(HeapTuple tuple);
+extern jobject pljava_Tuple_internalCreate(HeapTuple tuple, bool mustCopy);
+extern jobjectArray pljava_Tuple_createArray(
+	HeapTuple* tuples, jint size, bool mustCopy);
 
 /*
  * Return a java object at given index from a HeapTuple (with a best effort to
  * produce an object of class rqcls if it is not null).
  */
-extern jobject Tuple_getObject(
+extern jobject pljava_Tuple_getObject(
 	TupleDesc tupleDesc, HeapTuple tuple, int index, jclass rqcls);
+extern void pljava_Tuple_initialize(void);
 
 #ifdef __cplusplus
 }
