@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2018-2020 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -11,13 +11,15 @@
  */
 package org.postgresql.pljava.internal;
 
+import static java.lang.invoke.MethodHandles.lookup;
+import java.lang.invoke.VarHandle;
+
 import java.lang.ref.ReferenceQueue;
 import java.lang.ref.WeakReference;
 
 import java.sql.SQLException;
 
 import java.util.ArrayDeque;
-import static java.util.Arrays.asList;
 import static java.util.Arrays.copyOf;
 import java.util.Deque;
 import java.util.HashMap;
@@ -29,14 +31,18 @@ import java.util.Queue;
 
 import java.util.concurrent.CancellationException;
 import java.util.concurrent.ConcurrentLinkedQueue;
-import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.LongAdder;
 import static java.util.concurrent.locks.LockSupport.park;
 import static java.util.concurrent.locks.LockSupport.unpark;
 
+import java.util.function.Supplier;
+
 import static java.lang.management.ManagementFactory.getPlatformMBeanServer;
 import javax.management.ObjectName;
 import javax.management.JMException;
 
+import org.postgresql.pljava.mbeans.DualStateStatistics;
+
 /**
  * Base class for object state with corresponding Java and native components.
  *<p>
@@ -261,7 +267,7 @@ public abstract class DualState<T> extends WeakReference<T>
 	 * in selected places where it makes sense to do so.
 	 */
 	private static final ReferenceQueue<Object> s_releasedInstances =
-		new ReferenceQueue<Object>();
+		new ReferenceQueue<>();
 
 	/**
 	 * {@code DualState} objects that arrived on {@code s_releasedInstances}
@@ -274,7 +280,7 @@ public abstract class DualState<T> extends WeakReference<T>
 	 * the case doesn't suggest a need for anything more elaborate.
 	 */
 	private static final Deque<DualState<?>> s_deferredReleased =
-		new ArrayDeque<DualState<?>>();
+		new ArrayDeque<>();
 
 	/**
 	 * All instances in a non-transient native scope are added here upon
@@ -286,7 +292,7 @@ public abstract class DualState<T> extends WeakReference<T>
 	 * searching, and can be removed from this structure in O(1).
 	 */
 	private static final IdentityHashMap<DualState,DualState>
-		s_unscopedInstances = new IdentityHashMap<DualState,DualState>();
+		s_unscopedInstances = new IdentityHashMap<>();
 
 	/**
 	 * All native-scoped instances are added to this structure upon creation.
@@ -299,7 +305,7 @@ public abstract class DualState<T> extends WeakReference<T>
 	 * already at hand. The list head is of a dummy {@code DualState} subclass.
 	 */
 	private static final Map<Long,DualState.ListHead> s_scopedInstances =
-		new HashMap<Long,DualState.ListHead>();
+		new HashMap<>();
 
 	/** Backward link in per-resource-owner list. */
 	private DualState m_prev;
@@ -370,11 +376,13 @@ public abstract class DualState<T> extends WeakReference<T>
 
 			/**
 			 * Increment a thread-local count of pins for a DualState object.
-			 * @return true if there was already at least one pin counted for
+			 * @return null if there was already at least one pin counted for
 			 * the object (that is, no real pin will need to be taken; this is
-			 * a reentrant pin).
+			 * a reentrant pin); otherwise, a Supplier<Queue<Thread>> that can
+			 * supply a preallocated queue prepopulated with the current thread,
+			 * in case inflation is needed.
 			 */
-			boolean pin(DualState<?> s)
+			Supplier<Queue<Thread>> pin(DualState<?> s)
 			{
 				boolean result = false; // assume a real pin must be taken
 				Manager counts = get();
@@ -384,9 +392,19 @@ public abstract class DualState<T> extends WeakReference<T>
 					result = counts.hasPin(s);
 					pc = counts.push(s);
 				}
-				if ( 0 < pc.m_count ++ )
-					return true;
-				return result;
+				if ( 0 < pc.m_count ++  ||  result )
+					return null;
+				/*
+				 * Ensure that counts.m_protoWaiters contains a preallocated
+				 * queue with this thread already added to it, ready for
+				 * immediate use by a contended pin that needs to inflate.
+				 */
+				if ( null == counts.m_protoWaiters )
+				{
+					counts.m_protoWaiters = new ConcurrentLinkedQueue<>();
+					counts.m_protoWaiters.add(Thread.currentThread());
+				}
+				return counts;
 			}
 
 			/**
@@ -426,13 +444,22 @@ public abstract class DualState<T> extends WeakReference<T>
 		 * objects once pushed on it, for reuse, intended to produce less
 		 * observed garbage than the earlier straight use of ArrayDeque.
 		 */
-		static final class Manager
+		static final class Manager implements Supplier<Queue<Thread>>
 		{
 			private static final int INITIAL_SIZE = 4;
 			private static final int POOL_TARGET = 2;
 			private PinCount[] m_array = new PinCount [ INITIAL_SIZE ];
 			private int m_top = -1;
 			private int m_pooled = 0;
+			Queue<Thread> m_protoWaiters;
+
+			@Override
+			public Queue<Thread> get()
+			{
+				Queue<Thread> q = m_protoWaiters;
+				m_protoWaiters = null;
+				return q;
+			}
 
 			PinCount peek()
 			{
@@ -538,6 +565,18 @@ public abstract class DualState<T> extends WeakReference<T>
 		}
 	}
 
+	/**
+	 * {@code VarHandle} for applying atomic operations on the {@code m_state}
+	 * field.
+	 */
+	private static final VarHandle s_stateVH;
+
+	/**
+	 * {@code VarHandle} for applying atomic operations on the {@code m_waiters}
+	 * field.
+	 */
+	private static final VarHandle s_waitersVH;
+
 	/**
 	 * Bean to expose DualState allocation/release statistics to JMX management
 	 * tools.
@@ -545,13 +584,25 @@ public abstract class DualState<T> extends WeakReference<T>
 	private static final Statistics s_stats = new Statistics();
 
 	static {
+		try
+		{
+			s_stateVH =
+				lookup().findVarHandle(DualState.class, "m_state", int.class);
+			s_waitersVH = lookup().findVarHandle(DualState.class, "m_waiters",
+				Queue.class);
+		}
+		catch ( ReflectiveOperationException e )
+		{
+			throw new ExceptionInInitializerError(e);
+		}
+
 		try
 		{
 			ObjectName n = new ObjectName(
 				"org.postgresql.pljava:type=DualState,name=Statistics");
 			getPlatformMBeanServer().registerMBean(s_stats, n);
 		}
-		catch ( JMException e ) { }
+		catch ( JMException e ) { /* XXX */ }
 	}
 
 	/**
@@ -591,16 +642,16 @@ public abstract class DualState<T> extends WeakReference<T>
 	private static final int PINNERS_MASK    = 0x00001fff;
 
 	/** Lock state, also records whether native or Java release has occurred. */
-	private final AtomicInteger m_state;
+	private volatile int m_state = 0;
 
 	/** Threads waiting for pins pending release of lock. */
-	private final Queue<Thread> m_waiters;
+	private Queue<Thread> m_waiters;
 
 	/** True if argument is zero. */
 	static boolean z(int i) { return 0 == i; }
 
 	/**
-	 * Return the argument; convenient breakpoint targe for failed assertions.
+	 * Return the argument; convenient breakpoint target for failed assertions.
 	 */
 	static <T> T m(T detail)
 	{
@@ -636,9 +687,6 @@ public abstract class DualState<T> extends WeakReference<T>
 
 		m_resourceOwner = resourceOwner;
 
-		m_state = new AtomicInteger();
-		m_waiters = new ConcurrentLinkedQueue<Thread>();
-
 		assert Backend.threadMayEnterPG() : m("DualState construction");
 		/*
 		 * The following stanza publishes 'this' into one of the static data
@@ -681,7 +729,6 @@ public abstract class DualState<T> extends WeakReference<T>
 		super.clear();   // but nobody ever said for how long.
 		m_resourceOwner = resourceOwner;
 		m_prev = m_next = this;
-		m_state = null;
 		m_waiters = null;
 	}
 
@@ -756,12 +803,6 @@ public abstract class DualState<T> extends WeakReference<T>
 		javaStateUnreachable(nativeStateLive);
 	}
 
-	/**
-	 * Part of a pre-Java-9 attempt to achieve something like reachabilityFence;
-	 * see releaseFromJava.
-	 */
-	private static final List<?> s_nulls = asList((Object)null);
-
 	/**
 	 * What Java code will call to explicitly release this instance
 	 * (in the implementation of {@code close}, for example).
@@ -797,11 +838,7 @@ public abstract class DualState<T> extends WeakReference<T>
 		 *
 		 * Optimizing JIT could muddy the picture, though, by deciding the only
 		 * use of r1 here is to compare it to null, which could be moved ahead
-		 * of the clear(). In Java 9, that can be prevented explicitly by
-		 * placing reachabilityFence(r1) after the clear(). In the pre-Java-9
-		 * world, making the comparison to null somewhat less constant-foldable
-		 * and placing it on the far side of the atomic operations on m_state
-		 * will have to do, with some crossing of fingers.
+		 * of the clear(), in the absence of the reachabilityFence below.
 		 *
 		 * The m_state operations are the only ones with synchronizing effects,
 		 * so we had better delay our unconditional clear() until after we know
@@ -816,30 +853,20 @@ public abstract class DualState<T> extends WeakReference<T>
 		 */
 		T r1 = referent();
 
-		int s = 0; // start by assuming state is simple with no pins
-		int t;
-		for ( ;; )
-		{
-			t = s | JAVA_RELEASED;
-			if ( m_state.compareAndSet(s, t) )
-				break;
-			s = m_state.get();
-			if ( !z(s & JAVA_RELEASED) )
-				break; // someone else has beaten us to marking it released.
-		}
+		int s = (int)s_stateVH.getAndBitwiseOr(this, JAVA_RELEASED);
 		/*
 		 * The state is now marked JAVA_RELEASED; that flag may have been set
 		 * before us, or here now by us (the latter indicated by its absence
 		 * in s).
 		 *
-		 * Now to clear the referent, and find out if it was live at entry.
+		 * Now to find out if the referent was live at entry, and clear it.
 		 */
-		super.clear();
-		T r2 = referent();
-		boolean wasClearAtEntry = s_nulls.containsAll(asList(r1, r2));
 		boolean releaseFlagWasClear = z(s & JAVA_RELEASED);
+		boolean refWasClearAtEntry = null == r1;
+		super.clear();
+		reachabilityFence(r1);
 
-		if ( wasClearAtEntry )
+		if ( refWasClearAtEntry )
 		{
 			if ( releaseFlagWasClear )
 			{
@@ -905,7 +932,7 @@ public abstract class DualState<T> extends WeakReference<T>
 		 * So, spare the exception as long as the instance has in fact been
 		 * released.
 		 */
-		int s = m_state.get();
+		int s = (int)s_stateVH.get(this);
 		if ( z(s & JAVA_RELEASED) )
 			throw new UnsupportedOperationException(
 				"directly calling clear() on a DualState object is not " +
@@ -982,17 +1009,25 @@ public abstract class DualState<T> extends WeakReference<T>
 	 */
 	private final int _pin()
 	{
-		if ( s_pinCount.pin(this) )
+		/*
+		 * The test for a reentrant pin will indicate its result by returning
+		 * null (if the pin is reentrant and no further action is needed here)
+		 * or a queue supplier, which is ready to supply a preallocated queue
+		 * in case inflation is needed.
+		 */
+		Supplier<Queue<Thread>> qSupplier = s_pinCount.pin(this);
+
+		if ( null == qSupplier )
 			return 0; // reentrant pin, no need for sync effort
 
-		int s = m_state.incrementAndGet(); // be optimistic
+		int s = 1 + (int)s_stateVH.getAndAdd(this, 1); // be optimistic
 		if ( z(s & ~ PINNERS_MASK) )       // nothing in s but a pin count? ->
 			return 0;                        // ... uncontended win!
 		if ( !z(s & (NATIVE_RELEASED | JAVA_RELEASED)) )
 			return backoutPinBeforeEnqueue(s);
 		if ( !z(s & PINNERS_GUARD) )
 		{
-			m_state.decrementAndGet(); // recovery is questionable in this case
+			s = (int)s_stateVH.getAndAdd(this, -1); //recovery iffy in this case
 			s_pinCount.unpin(this);
 			throw new Error("DualState pin tracking capacity exceeded");
 		}
@@ -1013,8 +1048,42 @@ public abstract class DualState<T> extends WeakReference<T>
 		 */
 
 		Thread thr = Thread.currentThread();
-		m_waiters.add(thr);
+
+		/*
+		 * Observation shows contention is very rare, so m_waiters can be left
+		 * null for most DualState instances, and be 'inflated' by having a
+		 * queue installed when first needed. That requires a null check here.
+		 */
+		if ( null != m_waiters )
+			m_waiters.add(thr);
+		else
+		{
+			/*
+			 * We install the queue with a CAS on m_waiters, which is enough to
+			 * coordinate with any other thread trying to do this concurrently.
+			 * It is not enough to synchronize with unlock(), which uses only a
+			 * plain read on m_waiters. But it only does that after seeing our
+			 * upcoming modification to m_state, which this happens before.
+			 */
+			if ( s_waitersVH.compareAndSet(this, null, qSupplier.get()) )
+			{
+				/*
+				 * We successfully inflated. The queue obtained from get() above
+				 * already has this thread enqueued on it, so there is nothing
+				 * else to do here.
+				 */
+			}
+			else
+			{
+				/*
+				 * Somebody beat us to it. Their queue is just as good; use it.
+				 */
+				m_waiters.add(thr);
+			}
+		}
+
 		int t;
+		int u;
 		/*
 		 * Top-of-loop invariant, s has either MUTATOR_HOLDS or MUTATOR_WANTS,
 		 * and we're counted under PINNERS_MASK, but under WAITERS_MASK is where
@@ -1030,7 +1099,7 @@ public abstract class DualState<T> extends WeakReference<T>
 		 * the PINNERS count prevents a WANTS advancing to HOLDS, and also
 		 * blocks the final CAS in the release of a HOLDS.
 		 */
-		for ( ;; s = m_state.get() )
+		for ( ;; s = u )
 		{
 			t = s + (PINNERS_GUARD|PINNERS_MASK);
 			/*
@@ -1047,7 +1116,8 @@ public abstract class DualState<T> extends WeakReference<T>
 			}
 			if ( !z(t & MUTATOR_WANTS)  &&  z(t & PINNERS_MASK) )
 				t += MUTATOR_WANTS; // promote to MUTATOR_HOLDS, next bit left
-			if ( m_state.compareAndSet(s, t) )
+			u = (int)s_stateVH.compareAndExchange(this, s, t);
+			if ( s == u )
 				break;
 		}
 		if ( !z(t & MUTATOR_HOLDS)  &&  !z(s & MUTATOR_WANTS)) // promoted by us
@@ -1062,10 +1132,10 @@ public abstract class DualState<T> extends WeakReference<T>
 		{
 			if ( ! thr.isInterrupted() )
 				park(this);
-			s = m_state.get();
+			s = (int)s_stateVH.getVolatile(this);
 			if ( thr.isInterrupted()
 				|| !z(s & (NATIVE_RELEASED | JAVA_RELEASED)) )
-				return backoutPinAfterPark(s, t);
+				return backoutPinAfterPark(t);
 			if ( !z(s & MUTATOR_HOLDS) ) // can only be a spurious unpark
 				continue;
 			if ( z(s & MUTATOR_WANTS) )  // no HOLDS, no WANTS, so
@@ -1116,13 +1186,15 @@ public abstract class DualState<T> extends WeakReference<T>
 
 		int s = 1; // start by assuming state is simple with only one pinner, us
 		int t;
-		for ( ;; s = m_state.get() )
+		int u;
+		for ( ;; s = u )
 		{
 			assert 1 <= (s & PINNERS_MASK) : m("DualState unpin < 1 pinner");
 			t = s - 1;
 			if ( !z(t & MUTATOR_WANTS)  &&  z(t & PINNERS_MASK) )
 				t += MUTATOR_WANTS; // promote to MUTATOR_HOLDS, next bit left
-			if ( m_state.compareAndSet(s, t) )
+			u = (int)s_stateVH.compareAndExchange(this, s, t);
+			if ( s == u )
 				break;
 		}
 
@@ -1150,6 +1222,7 @@ public abstract class DualState<T> extends WeakReference<T>
 	/**
 	 * Back out an in-progress pin before our thread has been placed on the
 	 * queue.
+	 * @param s the most recently known state
 	 * @return the {@code NATIVE_RELEASED} and {@code JAVA_RELEASED} bits of
 	 * the state
 	 */
@@ -1157,13 +1230,15 @@ public abstract class DualState<T> extends WeakReference<T>
 	{
 		s_pinCount.unpin(this);
 		int t;
-		for ( ;; s = m_state.get() )
+		int u;
+		for ( ;; s = u )
 		{
 			assert 1 <= (s & PINNERS_MASK) : m("backoutPinBeforeEnqueue");
 			t = s - 1;
 			if ( !z(t & MUTATOR_WANTS)  &&  z(t & PINNERS_MASK) )
 				t += MUTATOR_WANTS; // promote to MUTATOR_HOLDS, next bit left
-			if ( m_state.compareAndSet(s, t) )
+			u = (int)s_stateVH.compareAndExchange(this, s, t);
+			if ( s == u )
 				break;
 		}
 		/*
@@ -1180,6 +1255,7 @@ public abstract class DualState<T> extends WeakReference<T>
 	 * Back out an in-progress pin after our thread has been placed on the
 	 * queue, but before success of the CAS that counts us under WAITERS_MASK
 	 * rather than PINNERS_MASK.
+	 * @param s the most recently known state
 	 * @return the {@code NATIVE_RELEASED} and {@code JAVA_RELEASED} bits of
 	 * the state
 	 */
@@ -1199,13 +1275,12 @@ public abstract class DualState<T> extends WeakReference<T>
 	 * either throws the {@code CancellationException} (if thread interruption
 	 * was the reason), or returns one or both of {@code NATIVE_RELEASED} or
 	 * {@code JAVA_RELEASED}; it never returns zero.
-	 * @param s the most recently fetched state
 	 * @param t prior state from before parking
 	 * @throws CancellationException if the reason was thread interruption
 	 * @return the {@code NATIVE_RELEASED} and {@code JAVA_RELEASED} bits of
 	 * the state
 	 */
-	private int backoutPinAfterPark(int s, int t)
+	private int backoutPinAfterPark(int t)
 	{
 		boolean wasHolds = !z(t & MUTATOR_HOLDS); // t, the pre-park state
 		/*
@@ -1213,12 +1288,7 @@ public abstract class DualState<T> extends WeakReference<T>
 		 * transitions by the mutator thread (WANTS to HOLDS, or HOLDS to
 		 * released) while we determine what to do next.
 		 */
-		for ( ;; s = m_state.get() )
-		{
-			t = s + 1;
-			if ( m_state.compareAndSet(s, t) )
-				break;
-		}
+		t = 1 + (int)s_stateVH.getAndAdd(this, 1);
 
 		/*
 		 * From the current state, determine whether our pin has in fact been
@@ -1251,13 +1321,7 @@ public abstract class DualState<T> extends WeakReference<T>
 			delta = 1 << WAITERS_SHIFT;
 			assert delta <= (t & WAITERS_MASK) : m("backoutPinAfterPark");
 		}
-		s = t;
-		for ( ;; s = m_state.get() )
-		{
-			t = s - delta;
-			if ( m_state.compareAndSet(s, t) )
-				break;
-		}
+		t = (int)s_stateVH.getAndAdd(this, - delta) - delta;
 
 		if ( mustUnpin )
 			unpin();
@@ -1328,6 +1392,7 @@ public abstract class DualState<T> extends WeakReference<T>
 
 		int s = upgrade ? 1 : 0; // to start, assume simple state, no other pins
 		int t;
+		int u;
 		int contended = 0;
 		for ( ;; )
 		{
@@ -1335,9 +1400,10 @@ public abstract class DualState<T> extends WeakReference<T>
 			if ( upgrade )
 				t += (PINNERS_GUARD|PINNERS_MASK); // hide my pin as a waiter
 			t |= ( !z(t & PINNERS_MASK) ? MUTATOR_WANTS : MUTATOR_HOLDS );
-			if ( m_state.compareAndSet(s, t) )
+			u = (int)s_stateVH.compareAndExchange(this, s, t);
+			if ( s == u )
 				break;
-			s = m_state.get();
+			s = u;
 			if ( !z(s & MUTATOR_HOLDS) ) // surprise! this is a reentrant call.
 				return s & (NATIVE_RELEASED | JAVA_RELEASED | MUTATOR_HOLDS);
 			if ( z(s & PINNERS_MASK) )
@@ -1347,7 +1413,7 @@ public abstract class DualState<T> extends WeakReference<T>
 		{
 			contended = 1;
 			park(this);
-			t = m_state.get();
+			t = (int)s_stateVH.getVolatile(this);
 		}
 		s_stats.lockContended(contended);
 		return t & (NATIVE_RELEASED | JAVA_RELEASED) | (upgrade? 1 : 0);
@@ -1371,6 +1437,7 @@ public abstract class DualState<T> extends WeakReference<T>
 	protected final void unlock(int s, boolean isNativeRelease)
 	{
 		int t;
+		int u;
 		if ( !z(s & MUTATOR_HOLDS) )
 		{
 			/*
@@ -1381,14 +1448,7 @@ public abstract class DualState<T> extends WeakReference<T>
 			 * else does.
 			 */
 			if ( isNativeRelease  &&  z(s & NATIVE_RELEASED) )
-			{
-				for ( ;; s = m_state.get() )
-				{
-					t = s | NATIVE_RELEASED;
-					if ( m_state.compareAndSet(s, t) )
-						break;
-				}
-			}
+				u = (int)s_stateVH.getAndBitwiseOr(this, NATIVE_RELEASED);
 			return;
 		}
 
@@ -1415,9 +1475,10 @@ public abstract class DualState<T> extends WeakReference<T>
 			 * bits under WAITERS_MASK where they belong. Then trust the queue.
 			 */
 			s &= ~ PINNERS_MASK;
-			if ( m_state.compareAndSet(s, t) )
+			u = (int)s_stateVH.compareAndExchange(this, s, t);
+			if ( s == u )
 				break;
-			s = m_state.get();
+			s = u;
 			assert !z(s & MUTATOR_HOLDS) : m("DualState mispaired unlock");
 			/*
 			 * Most of our CAS spins in this class require only that no other
@@ -1427,10 +1488,10 @@ public abstract class DualState<T> extends WeakReference<T>
 			 * other thread to enqueue itself and move its bit out of the way.
 			 * That's still a very short spin, unless we are short of CPUs and
 			 * actually competing with the thread we're waiting for. Hence
-			 * a yield seems prudent, if pinner count is the reason we spin.
+			 * onSpinWait seems prudent, if pinner count is the reason we spin.
 			 */
 			if ( !z(s & PINNERS_MASK) )
-				Thread.yield();
+				Thread.onSpinWait();
 		}
 		/*
 		 * It's good to be the only thread allowed to mutate. Nobody else will
@@ -1440,6 +1501,12 @@ public abstract class DualState<T> extends WeakReference<T>
 		t &= PINNERS_MASK; // should equal the number of threads on the queue
 		if ( upgrade ) // my pin bit was stashed as a waiter, but nothing queued
 			-- t;
+		/*
+		 * If no waiters (t is zero), we are done. Don't bother comparing zero
+		 * to the queue size; inflation may not have supplied a queue yet.
+		 */
+		if ( 0 == t )
+			return;
 		s_stats.pinContended(t);
 		Thread thr;
 		while ( null != (thr = m_waiters.poll()) )
@@ -1471,7 +1538,7 @@ public abstract class DualState<T> extends WeakReference<T>
 		int s = 1; // must be: quiescent (our pin only), unreleased
 		int t = NATIVE_RELEASED | JAVA_RELEASED | MUTATOR_HOLDS
 				| 1 << WAITERS_SHIFT;
-		if ( ! m_state.compareAndSet(s, t) )
+		if ( ! (boolean)s_stateVH.compareAndSet(this, s, t) )
 			throw new SQLException(
 				"Attempt by PostgreSQL to adopt a released or non-quiescent " +
 				"Java object");
@@ -1504,7 +1571,7 @@ public abstract class DualState<T> extends WeakReference<T>
 		 */
 		nativeStateReleased(false);
 
-		if ( ! m_state.compareAndSet(s, t) )
+		if ( ! (boolean)s_stateVH.compareAndSet(this, s, t) )
 			throw new SQLException("Release failed while adopting Java object");
 
 		/*
@@ -1616,10 +1683,11 @@ public abstract class DualState<T> extends WeakReference<T>
 	{
 		Class<?> c = (null == o ? this : o).getClass();
 		String cn = c.getCanonicalName();
-		int pnl = c.getPackage().getName().length();
+		int pnl = c.getPackageName().length();
 		return String.format("%s owner:%x %s",
 			cn.substring(1 + pnl), m_resourceOwner,
-			z(m_state.get() & NATIVE_RELEASED) ? "fresh" : "stale");
+			z((int)s_stateVH.getVolatile(this) & NATIVE_RELEASED)
+				? "fresh" : "stale");
 	}
 
 	/**
@@ -1714,7 +1782,7 @@ public abstract class DualState<T> extends WeakReference<T>
 				else if ( null == (s = (DualState)s_releasedInstances.poll()) )
 					break;
 
-				int state = s.m_state.get();
+				int state = (int)s_stateVH.getVolatile(s);
 				if ( !z((PINNERS_MASK | WAITERS_MASK) & state) )
 				{
 					s_deferredReleased.add(s);
@@ -2138,126 +2206,170 @@ public abstract class DualState<T> extends WeakReference<T>
 	 * Bean exposing some {@code DualState} allocation and lifecycle statistics
 	 * for viewing in a JMX management client.
 	 */
-	public static interface StatisticsMBean
+	static class Statistics implements DualStateStatistics
 	{
-		long getConstructed();
-		long getEnlistedScoped();
-		long getEnlistedUnscoped();
-		long getDelistedScoped();
-		long getDelistedUnscoped();
-		long getJavaUnreachable();
-		long getJavaReleased();
-		long getNativeReleased();
-		long getResourceOwnerPasses();
-		long getReferenceQueuePasses();
-		long getReferenceQueueItems();
-		long getContendedLocks();
-		long getContendedPins();
-		long getRepeatedlyDeferred();
-		int getGcReleaseRaces();
-		int getReleaseReleaseRaces();
-	}
+		public long getConstructed()
+		{
+			return constructed.sum();
+		}
 
-	static class Statistics implements StatisticsMBean
-	{
-		public long getConstructed()           { return constructed; }
-		public long getEnlistedScoped()        { return enlistedScoped; }
-		public long getEnlistedUnscoped()      { return enlistedUnscoped; }
-		public long getDelistedScoped()        { return delistedScoped; }
-		public long getDelistedUnscoped()      { return delistedUnscoped; }
-		public long getJavaUnreachable()       { return javaUnreachable; }
-		public long getJavaReleased()          { return javaReleased; }
-		public long getNativeReleased()        { return nativeReleased; }
-		public long getResourceOwnerPasses()   { return resourceOwnerPasses; }
-		public long getReferenceQueuePasses()  { return referenceQueuePasses; }
-		public long getReferenceQueueItems()   { return referenceQueueItems; }
-		public long getContendedLocks()        { return contendedLocks; }
-		public long getContendedPins()         { return contendedPins; }
-		public long getRepeatedlyDeferred()    { return repeatedlyDeferred; }
-		public int  getGcReleaseRaces()        { return gcRelRaces.get(); }
-		public int  getReleaseReleaseRaces()   { return relRelRaces.get(); }
+		public long getEnlistedScoped()
+		{
+			return enlistedScoped.sum();
+		}
 
-		private long constructed = 0L;
-		private long enlistedScoped = 0L;
-		private long enlistedUnscoped = 0L;
-		private long delistedScoped = 0L;
-		private long delistedUnscoped = 0L;
-		private long javaUnreachable = 0L;
-		private long javaReleased = 0L;
-		private long nativeReleased = 0L;
-		private long resourceOwnerPasses = 0L;
-		private long referenceQueuePasses = 0L;
-		private long referenceQueueItems = 0L;
-		private long contendedLocks = 0L;
-		private long contendedPins = 0L;
-		private long repeatedlyDeferred = 0L;
-		private AtomicInteger  gcRelRaces = new AtomicInteger();
-		private AtomicInteger relRelRaces = new AtomicInteger();
+		public long getEnlistedUnscoped()
+		{
+			return enlistedUnscoped.sum();
+		}
+
+		public long getDelistedScoped()
+		{
+			return delistedScoped.sum();
+		}
+
+		public long getDelistedUnscoped()
+		{
+			return delistedUnscoped.sum();
+		}
+
+		public long getJavaUnreachable()
+		{
+			return javaUnreachable.sum();
+		}
+
+		public long getJavaReleased()
+		{
+			return javaReleased.sum();
+		}
+
+		public long getNativeReleased()
+		{
+			return nativeReleased.sum();
+		}
+
+		public long getResourceOwnerPasses()
+		{
+			return resourceOwnerPasses.sum();
+		}
+
+		public long getReferenceQueuePasses()
+		{
+			return referenceQueuePasses.sum();
+		}
+
+		public long getReferenceQueueItems()
+		{
+			return referenceQueueItems.sum();
+		}
+
+		public long getContendedLocks()
+		{
+			return contendedLocks.sum();
+		}
+
+		public long getContendedPins()
+		{
+			return contendedPins.sum();
+		}
+
+		public long getRepeatedlyDeferred()
+		{
+			return repeatedlyDeferred.sum();
+		}
+
+		public long getGcReleaseRaces()
+		{
+			return gcRelRaces.sum();
+		}
+
+		public long getReleaseReleaseRaces()
+		{
+			return relRelRaces.sum();
+		}
+
+
+		private LongAdder          constructed = new LongAdder();
+		private LongAdder       enlistedScoped = new LongAdder();
+		private LongAdder     enlistedUnscoped = new LongAdder();
+		private LongAdder       delistedScoped = new LongAdder();
+		private LongAdder     delistedUnscoped = new LongAdder();
+		private LongAdder      javaUnreachable = new LongAdder();
+		private LongAdder         javaReleased = new LongAdder();
+		private LongAdder       nativeReleased = new LongAdder();
+		private LongAdder  resourceOwnerPasses = new LongAdder();
+		private LongAdder referenceQueuePasses = new LongAdder();
+		private LongAdder  referenceQueueItems = new LongAdder();
+		private LongAdder       contendedLocks = new LongAdder();
+		private LongAdder        contendedPins = new LongAdder();
+		private LongAdder   repeatedlyDeferred = new LongAdder();
+		private LongAdder           gcRelRaces = new LongAdder();
+		private LongAdder          relRelRaces = new LongAdder();
 
 		final void construct(long scoped)
 		{
-			++ constructed;
-			enlistedScoped += scoped;
-			enlistedUnscoped += 1L - scoped;
+			constructed.increment();
+			enlistedScoped.add(scoped);
+			enlistedUnscoped.add(1L - scoped);
 		}
 
 		final void resourceOwnerPoll(long released, long total)
 		{
-			++ resourceOwnerPasses;
-			nativeReleased += released;
-			delistedScoped += total;
+			resourceOwnerPasses.increment();
+			nativeReleased.add(released);
+			delistedScoped.add(total);
 		}
 
 		final void javaRelease(long scoped, long unscoped)
 		{
-			++ javaReleased;
-			delistedScoped += scoped;
-			delistedUnscoped += unscoped;
+			javaReleased.increment();
+			delistedScoped.add(scoped);
+			delistedUnscoped.add(unscoped);
 		}
 
 		final void referenceQueueDrain(
 			long unreachable, long release, long total, long reDefer)
 		{
-			++ referenceQueuePasses;
-			referenceQueueItems += total;
-			javaUnreachable += unreachable;
-			javaReleased += release;
-			repeatedlyDeferred += reDefer;
+			referenceQueuePasses.increment();
+			referenceQueueItems.add(total);
+			javaUnreachable.add(unreachable);
+			javaReleased.add(release);
+			repeatedlyDeferred.add(reDefer);
 		}
 
 		final void delistScoped()
 		{
-			++ delistedScoped;
+			delistedScoped.increment();
 		}
 
 		final void delistUnscoped()
 		{
-			++ delistedUnscoped;
+			delistedUnscoped.increment();
 		}
 
 		final void javaRelease()
 		{
-			++ javaReleased;
+			javaReleased.increment();
 		}
 
 		final void lockContended(int n)
 		{
-			contendedLocks += n;
+			contendedLocks.add(n);
 		}
 
 		final void pinContended(int n)
 		{
-			contendedPins += n;
+			contendedPins.add(n);
 		}
 
 		final void gcReleaseRace()
 		{
-			gcRelRaces.getAndIncrement();
+			gcRelRaces.increment();
 		}
 
 		final void releaseReleaseRace()
 		{
-			gcRelRaces.getAndIncrement();
+			gcRelRaces.increment();
 		}
 	}
 }
