@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2022 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2025 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -19,6 +19,8 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.HashMap;
+import static java.util.Objects.requireNonNull;
+import java.util.Properties;
 
 import org.postgresql.pljava.ObjectPool;
 import org.postgresql.pljava.PooledObject;
@@ -49,8 +51,16 @@ public class Session implements org.postgresql.pljava.Session
 		return Holder.INSTANCE;
 	}
 
+	private final Properties m_properties;
+
 	private Session()
 	{
+		/*
+		 * This strategy assumes that no user code will request a Session
+		 * instance until after InstallHelper has poked the frozen properties
+		 * into s_properties.
+		 */
+		m_properties = requireNonNull(s_properties);
 	}
 
 	private static class Holder
@@ -58,8 +68,11 @@ public class Session implements org.postgresql.pljava.Session
 		static final Session INSTANCE = new Session();
 	}
 
-	@SuppressWarnings("removal")
-	private final TransactionalMap m_attributes = new TransactionalMap(new HashMap());
+	/**
+	 * An unmodifiable defensive copy of the Java system properties that will be
+	 * put here by InstallHelper via package access at startup.
+	 */
+	static Properties s_properties;
 
 	/**
 	 * The Java charset corresponding to the server encoding, or null if none
@@ -67,6 +80,12 @@ public class Session implements org.postgresql.pljava.Session
 	 */
 	static Charset s_serverCharset;
 
+	@Override
+	public Properties frozenSystemProperties()
+	{
+		return m_properties;
+	}
+
 	/**
 	 * A static method (not part of the API-exposed Session interface) by which
 	 * pljava implementation classes can get hold of the server charset without
@@ -86,6 +105,9 @@ public class Session implements org.postgresql.pljava.Session
 		return s_serverCharset;
 	}
 
+	@SuppressWarnings("removal")
+	private final TransactionalMap m_attributes = new TransactionalMap(new HashMap());
+
 	/**
 	 * Adds the specified listener to the list of listeners that will
 	 * receive transactional events.
