@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2022 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -76,7 +76,7 @@ class SubXactListener
 	throws SQLException
 	{
 		Target target = s_refs[eventIndex];
-		Session session = Backend.getSession();
+		Session session = org.postgresql.pljava.internal.Session.provider();
 
 		// Take a snapshot. Handlers might unregister during event processing
 		for ( Invocable<SavepointListener> listener :
