@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2018 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -71,10 +71,7 @@ public abstract class AbstractResultSet implements ResultSet
 		return this.getBigDecimal(this.findColumn(columnName));
 	}
 
-	/**
-	 * @deprecated
-	 */
-	@Override
+	@SuppressWarnings("deprecation") @Override
 	public BigDecimal getBigDecimal(String columnName, int scale)
 	throws SQLException
 	{
@@ -180,7 +177,7 @@ public abstract class AbstractResultSet implements ResultSet
 	}
 
 	@Override
-	public Object getObject(String columnName, Map map)
+	public Object getObject(String columnName, Map<String,Class<?>> map)
 	throws SQLException
 	{
 		return this.getObject(this.findColumn(columnName), map);
@@ -235,10 +232,7 @@ public abstract class AbstractResultSet implements ResultSet
 		return this.getTimestamp(this.findColumn(columnName), cal);
 	}
 
-	/**
-	 * @deprecated
-	 */
-	@Override
+	@SuppressWarnings("deprecation") @Override
 	public InputStream getUnicodeStream(String columnName)
 	throws SQLException
 	{
@@ -878,15 +872,12 @@ public abstract class AbstractResultSet implements ResultSet
 
 	// ************************************************************
 	// Implementation of JDBC 4.1 methods.
-	// Add @Override here once Java back horizon advances to 7.
 	// ************************************************************
 
+	@Override
 	public <T> T getObject(String columnName, Class<T> type)
 	throws SQLException
 	{
 		return this.getObject(this.findColumn(columnName), type);
 	}
-
-	public abstract <T> T getObject(int columnIndex, Class<T> type)
-	throws SQLException; // placeholder; remove when Java back horizon >= 7
 }
