@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2025 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -848,7 +848,9 @@ public class SPIConnection implements Connection
 		{
 			try
 			{
-				return (T)new URL((String)value);
+				@SuppressWarnings("deprecation") // PL/Java major rev or forever
+				T result = (T)new URL((String)value);
+				return result;
 			}
 			catch(MalformedURLException e)
 			{
