@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2020 Tada AB and other contributors, as listed below.
  * Copyright (c) 2010, 2011 PostgreSQL Global Development Group
  *
  * All rights reserved. This program and the accompanying materials
@@ -18,9 +18,9 @@ import java.io.BufferedReader;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.Reader;
-import java.io.UnsupportedEncodingException;
 import java.math.BigDecimal;
 import java.net.URL;
+import static java.nio.charset.StandardCharsets.US_ASCII;
 import java.sql.Array;
 import java.sql.Blob;
 import java.sql.Clob;
@@ -41,6 +41,7 @@ import org.postgresql.pljava.internal.Tuple;
 import org.postgresql.pljava.internal.TupleDesc;
 
 /**
+ * Implementation of {@link SQLOutput} for the case of a composite data type.
  * @author Thomas Hallgren
  */
 public class SQLOutputToTuple implements SQLOutput
@@ -85,15 +86,8 @@ public class SQLOutputToTuple implements SQLOutput
 
 	public void writeAsciiStream(InputStream value) throws SQLException
 	{
-		try
-		{
-			Reader rdr = new BufferedReader(new InputStreamReader(value, "US-ASCII"));
-			writeClob(new ClobValue(rdr, ClobValue.getReaderLength(rdr)));
-		}
-		catch(UnsupportedEncodingException e)
-		{
-			throw new SQLException(e.toString());
-		}
+		Reader rdr = new BufferedReader(new InputStreamReader(value, US_ASCII));
+		writeClob(new ClobValue(rdr, ClobValue.getReaderLength(rdr)));
 	}
 
 	public void writeBigDecimal(BigDecimal value) throws SQLException
