@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2021 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2025 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -130,7 +130,7 @@ public class Loader extends ClassLoader
 			return entryURL(m_entryIds[m_top++]);
 		}
 	}
-	private static final Identifier.Simple PUBLIC_SCHEMA =
+	public static final Identifier.Simple PUBLIC_SCHEMA =
 		Identifier.Simple.fromCatalog("public");
 
 	private static final Map<Identifier.Simple, ClassLoader>
@@ -229,6 +229,7 @@ public class Loader extends ClassLoader
 			{
 				while(rs.next())
 				{
+					@SuppressWarnings("deprecation") // until PL/Java major rev
 					URL jarUrl = new URL("sqlj:" + rs.getString(2));
 					CodeSource cs = new CodeSource(jarUrl, (CodeSigner[])null);
 
@@ -353,12 +354,14 @@ public class Loader extends ClassLoader
 	{
 		try
 		{
-			return doPrivileged(() -> new URL(
+			@SuppressWarnings("deprecation") // Java >= 20: URL.of(uri,handler)
+			URL u = doPrivileged(() -> new URL(
 					"dbf",
 					"localhost",
 					-1,
 					"/" + entryId,
 					EntryStreamHandler.getInstance()));
+			return u;
 		}
 		catch(MalformedURLException e)
 		{
